@@ -13,27 +13,29 @@
 (import '(java.io Writer))
 
 (def
- #^{:doc "*print-length* controls how many items of each collection the
+ ^{:doc "*print-length* controls how many items of each collection the
   printer will print. If it is bound to logical false, there is no
   limit. Otherwise, it must be bound to an integer indicating the maximum
   number of items of each collection to print. If a collection contains
   more items, the printer will print items up to the limit followed by
   '...' to represent the remaining items. The root binding is nil
-  indicating no limit."}
+  indicating no limit."
+   :added "1.0"}
  *print-length* nil)
 
 (def
- #^{:doc "*print-level* controls how many levels deep the printer will
+ ^{:doc "*print-level* controls how many levels deep the printer will
   print nested objects. If it is bound to logical false, there is no
   limit. Otherwise, it must be bound to an integer indicating the maximum
   level to print. Each argument to print is at level 0; if an argument is a
   collection, its items are at level 1; and so on. If an object is a
   collection and is at a level greater than or equal to the value bound to
   *print-level*, the printer prints '#' to represent it. The root binding
-  is nil indicating no limit."}
+  is nil indicating no limit."
+   :added "1.0"}
 *print-level* nil)
 
-(defn- print-sequential [#^String begin, print-one, #^String sep, #^String end, sequence, #^Writer w]
+(defn- print-sequential [^String begin, print-one, ^String sep, ^String end, sequence, ^Writer w]
   (binding [*print-level* (and (not *print-dup*) *print-level* (dec *print-level*))]
     (if (and *print-level* (neg? *print-level*))
       (.write w "#")
@@ -57,113 +59,109 @@
                 (recur xs)))))
         (.write w end)))))
 
-(defn- print-meta [o, #^Writer w]
+(defn- print-meta [o, ^Writer w]
   (when-let [m (meta o)]
     (when (and (pos? (count m))
                (or *print-dup*
                    (and *print-meta* *print-readably*)))
-      (.write w "#^")
+      (.write w "^")
       (if (and (= (count m) 1) (:tag m))
           (pr-on (:tag m) w)
           (pr-on m w))
       (.write w " "))))
 
-(defmethod print-method :default [o, #^Writer w]
+(defmethod print-method :default [o, ^Writer w]
   (print-method (vary-meta o #(dissoc % :type)) w))
 
-(defmethod print-method nil [o, #^Writer w]
+(defmethod print-method nil [o, ^Writer w]
   (.write w "nil"))
 
 (defmethod print-dup nil [o w] (print-method o w))
 
-(defn print-ctor [o print-args #^Writer w]
+(defn print-ctor [o print-args ^Writer w]
   (.write w "#=(")
-  (.write w (.getName #^Class (class o)))
+  (.write w (.getName ^Class (class o)))
   (.write w ". ")
   (print-args o w)
   (.write w ")"))
 
-(defmethod print-method Object [o, #^Writer w]
+(defmethod print-method Object [o, ^Writer w]
   (.write w "#<")
   (.write w (.getSimpleName (class o)))
   (.write w " ")
   (.write w (str o))
   (.write w ">"))
 
-(defmethod print-method clojure.lang.Keyword [o, #^Writer w]
+(defmethod print-method clojure.lang.Keyword [o, ^Writer w]
   (.write w (str o)))
 
 (defmethod print-dup clojure.lang.Keyword [o w] (print-method o w))
 
-(defmethod print-method Number [o, #^Writer w]
+(defmethod print-method Number [o, ^Writer w]
   (.write w (str o)))
 
-(defmethod print-dup Number [o, #^Writer w]
+(defmethod print-dup Number [o, ^Writer w]
   (print-ctor o
               (fn [o w]
                   (print-dup (str o) w))
               w))
 
-(defmethod print-dup clojure.lang.Fn [o, #^Writer w]
+(defmethod print-dup clojure.lang.Fn [o, ^Writer w]
   (print-ctor o (fn [o w]) w))
 
 (prefer-method print-dup clojure.lang.IPersistentCollection clojure.lang.Fn)
 (prefer-method print-dup java.util.Map clojure.lang.Fn)
 (prefer-method print-dup java.util.Collection clojure.lang.Fn)
 
-(defmethod print-method Boolean [o, #^Writer w]
+(defmethod print-method Boolean [o, ^Writer w]
   (.write w (str o)))
 
 (defmethod print-dup Boolean [o w] (print-method o w))
 
-(defn print-simple [o, #^Writer w]
+(defn print-simple [o, ^Writer w]
   (print-meta o w)
   (.write w (str o)))
 
-(defmethod print-method clojure.lang.Symbol [o, #^Writer w]
+(defmethod print-method clojure.lang.Symbol [o, ^Writer w]
   (print-simple o w))
 
 (defmethod print-dup clojure.lang.Symbol [o w] (print-method o w))
 
-(defmethod print-method clojure.lang.Var [o, #^Writer w]
+(defmethod print-method clojure.lang.Var [o, ^Writer w]
   (print-simple o w))
 
-(defmethod print-dup clojure.lang.Var [#^clojure.lang.Var o, #^Writer w]
+(defmethod print-dup clojure.lang.Var [^clojure.lang.Var o, ^Writer w]
   (.write w (str "#=(var " (.name (.ns o)) "/" (.sym o) ")")))
 
-(defmethod print-method clojure.lang.ISeq [o, #^Writer w]
+(defmethod print-method clojure.lang.ISeq [o, ^Writer w]
   (print-meta o w)
   (print-sequential "(" pr-on " " ")" o w))
 
 (defmethod print-dup clojure.lang.ISeq [o w] (print-method o w))
 (defmethod print-dup clojure.lang.IPersistentList [o w] (print-method o w))
-(prefer-method print-method clojure.lang.IPersistentList clojure.lang.ISeq)
-(prefer-method print-dup clojure.lang.IPersistentList clojure.lang.ISeq)
 (prefer-method print-method clojure.lang.ISeq clojure.lang.IPersistentCollection)
 (prefer-method print-dup clojure.lang.ISeq clojure.lang.IPersistentCollection)
 (prefer-method print-method clojure.lang.ISeq java.util.Collection)
 (prefer-method print-dup clojure.lang.ISeq java.util.Collection)
 
-(defmethod print-method clojure.lang.IPersistentList [o, #^Writer w]
-  (print-meta o w)
-  (print-sequential "(" print-method " " ")" o w))
 
 
-(defmethod print-dup java.util.Collection [o, #^Writer w]
+(defmethod print-dup java.util.Collection [o, ^Writer w]
  (print-ctor o #(print-sequential "[" print-dup " " "]" %1 %2) w))
 
-(defmethod print-dup clojure.lang.IPersistentCollection [o, #^Writer w]
+(defmethod print-dup clojure.lang.IPersistentCollection [o, ^Writer w]
   (print-meta o w)
   (.write w "#=(")
-  (.write w (.getName #^Class (class o)))
+  (.write w (.getName ^Class (class o)))
   (.write w "/create ")
   (print-sequential "[" print-dup " " "]" o w)
   (.write w ")"))
 
 (prefer-method print-dup clojure.lang.IPersistentCollection java.util.Collection)
 
-(def #^{:tag String 
-        :doc "Returns escape string for char or nil if none"}
+(def ^{:tag String 
+       :doc "Returns escape string for char or nil if none"
+       :added "1.0"}
   char-escape-string
     {\newline "\\n"
      \tab  "\\t"
@@ -173,7 +171,7 @@
      \formfeed "\\f"
      \backspace "\\b"})
 
-(defmethod print-method String [#^String s, #^Writer w]
+(defmethod print-method String [^String s, ^Writer w]
   (if (or *print-dup* *print-readably*)
     (do (.append w \")
       (dotimes [n (count s)]
@@ -186,27 +184,27 @@
 
 (defmethod print-dup String [s w] (print-method s w))
 
-(defmethod print-method clojure.lang.IPersistentVector [v, #^Writer w]
+(defmethod print-method clojure.lang.IPersistentVector [v, ^Writer w]
   (print-meta v w)
   (print-sequential "[" pr-on " " "]" v w))
 
 (defn- print-map [m print-one w]
   (print-sequential 
    "{"
-   (fn [e  #^Writer w] 
+   (fn [e  ^Writer w] 
      (do (print-one (key e) w) (.append w \space) (print-one (val e) w)))
    ", "
    "}"
    (seq m) w))
 
-(defmethod print-method clojure.lang.IPersistentMap [m, #^Writer w]
+(defmethod print-method clojure.lang.IPersistentMap [m, ^Writer w]
   (print-meta m w)
   (print-map m pr-on w))
 
-(defmethod print-dup java.util.Map [m, #^Writer w]
+(defmethod print-dup java.util.Map [m, ^Writer w]
   (print-ctor m #(print-map (seq %1) print-dup %2) w))
 
-(defmethod print-dup clojure.lang.IPersistentMap [m, #^Writer w]
+(defmethod print-dup clojure.lang.IPersistentMap [m, ^Writer w]
   (print-meta m w)
   (.write w "#=(")
   (.write w (.getName (class m)))
@@ -216,12 +214,13 @@
 
 (prefer-method print-dup clojure.lang.IPersistentCollection java.util.Map)
 
-(defmethod print-method clojure.lang.IPersistentSet [s, #^Writer w]
+(defmethod print-method clojure.lang.IPersistentSet [s, ^Writer w]
   (print-meta s w)
   (print-sequential "#{" pr-on " " "}" (seq s) w))
 
-(def #^{:tag String 
-        :doc "Returns name string for char or nil if none"} 
+(def ^{:tag String 
+       :doc "Returns name string for char or nil if none"
+       :added "1.0"} 
  char-name-string
    {\newline "newline"
     \tab "tab"
@@ -230,7 +229,7 @@
     \formfeed "formfeed"
     \return "return"})
 
-(defmethod print-method java.lang.Character [#^Character c, #^Writer w]
+(defmethod print-method java.lang.Character [^Character c, ^Writer w]
   (if (or *print-dup* *print-readably*)
     (do (.append w \\)
         (let [n (char-name-string c)]
@@ -258,14 +257,14 @@
    Byte/TYPE "Byte/TYPE"
    Short/TYPE "Short/TYPE"})
 
-(defmethod print-method Class [#^Class c, #^Writer w]
+(defmethod print-method Class [^Class c, ^Writer w]
   (.write w (.getName c)))
 
-(defmethod print-dup Class [#^Class c, #^Writer w]
+(defmethod print-dup Class [^Class c, ^Writer w]
   (cond
     (.isPrimitive c) (do
                        (.write w "#=(identity ")
-                       (.write w #^String (primitives-classnames c))
+                       (.write w ^String (primitives-classnames c))
                        (.write w ")"))
     (.isArray c) (do
                    (.write w "#=(java.lang.Class/forName \"")
@@ -275,17 +274,17 @@
             (.write w "#=")
             (.write w (.getName c)))))
 
-(defmethod print-method java.math.BigDecimal [b, #^Writer w]
+(defmethod print-method java.math.BigDecimal [b, ^Writer w]
   (.write w (str b))
   (.write w "M"))
 
-(defmethod print-method java.util.regex.Pattern [p #^Writer w]
+(defmethod print-method java.util.regex.Pattern [p ^Writer w]
   (.write w "#\"")
-  (loop [[#^Character c & r :as s] (seq (.pattern #^java.util.regex.Pattern p))
+  (loop [[^Character c & r :as s] (seq (.pattern ^java.util.regex.Pattern p))
          qmode false]
     (when s
       (cond
-        (= c \\) (let [[#^Character c2 & r2] r]
+        (= c \\) (let [[^Character c2 & r2] r]
                    (.append w \\)
                    (.append w c2)
                    (if qmode
@@ -301,17 +300,21 @@
                    (recur r qmode)))))
   (.append w \"))
 
-(defmethod print-dup java.util.regex.Pattern [p #^Writer w] (print-method p w))
+(defmethod print-dup java.util.regex.Pattern [p ^Writer w] (print-method p w))
 
-(defmethod print-dup clojure.lang.Namespace [#^clojure.lang.Namespace n #^Writer w]
+(defmethod print-dup clojure.lang.Namespace [^clojure.lang.Namespace n ^Writer w]
   (.write w "#=(find-ns ")
   (print-dup (.name n) w)
   (.write w ")"))
 
-(defmethod print-method clojure.lang.IDeref [o #^Writer w]
-  (print-sequential (format "#<%s@%x: "
+(defmethod print-method clojure.lang.IDeref [o ^Writer w]
+  (print-sequential (format "#<%s@%x%s: "
                             (.getSimpleName (class o))
-                            (System/identityHashCode o))
+                            (System/identityHashCode o)
+                            (if (and (instance? clojure.lang.Agent o)
+                                     (agent-error o))
+                              " FAILED"
+                              ""))
                     pr-on, "", ">", (list (if (and (future? o) (not (future-done? o))) :pending @o)), w))
 
-(def #^{:private true} print-initialized true)
+(def ^{:private true} print-initialized true)
