@@ -15,7 +15,7 @@
 ;; contributions and suggestions.
 
 (ns 
-  #^{:author "Stuart Sierra, with contributions and suggestions by 
+  ^{:author "Stuart Sierra, with contributions and suggestions by 
   Chas Emerick, Allen Rohner, and Stuart Halloway",
      :doc "A unit testing framework.
 
@@ -193,7 +193,7 @@
    All the test reporting functions write to the var *test-out*.  By
    default, this is the same as *out*, but you can rebind it to any
    PrintWriter.  For example, it could be a file opened with
-   clojure.contrib.duck-streams/writer.
+   clojure.java.io/writer.
 
 
    EXTENDING TEST-IS (ADVANCED)
@@ -237,15 +237,17 @@
 ;;; USER-MODIFIABLE GLOBALS
 
 (defonce
-  #^{:doc "True by default.  If set to false, no test functions will
+  ^{:doc "True by default.  If set to false, no test functions will
    be created by deftest, set-test, or with-test.  Use this to omit
-   tests when compiling or loading production code."}
+   tests when compiling or loading production code."
+    :added "1.1"}
   *load-tests* true)
 
 (def
- #^{:doc "The maximum depth of stack traces to print when an Exception
+ ^{:doc "The maximum depth of stack traces to print when an Exception
   is thrown during a test.  Defaults to nil, which means print the 
-  complete stack trace."}
+  complete stack trace."
+   :added "1.1"}
  *stack-trace-depth* nil)
 
 
@@ -264,27 +266,32 @@
 
 (defmacro with-test-out
   "Runs body with *out* bound to the value of *test-out*."
+  {:added "1.1"}
   [& body]
   `(binding [*out* *test-out*]
      ~@body))
 
-
-
 ;;; UTILITIES FOR REPORTING FUNCTIONS
 
 (defn file-position
   "Returns a vector [filename line-number] for the nth call up the
-  stack."
+  stack.
+
+  Deprecated in 1.2: The information needed for test reporting is
+  now on :file and :line keys in the result map."
+  {:added "1.1"
+   :deprecated "1.2"}
   [n]
-  (let [#^StackTraceElement s (nth (.getStackTrace (new java.lang.Throwable)) n)]
+  (let [^StackTraceElement s (nth (.getStackTrace (new java.lang.Throwable)) n)]
     [(.getFileName s) (.getLineNumber s)]))
 
 (defn testing-vars-str
   "Returns a string representation of the current test.  Renders names
   in *testing-vars* as a list, then the source file and line of
   current assertion."
-  []
-  (let [[file line] (file-position 4)]
+  {:added "1.1"}
+  [m]
+  (let [{:keys [file line]} m]
     (str
      ;; Uncomment to include namespace in failure report:
      ;;(ns-name (:ns (meta (first *testing-vars*)))) "/ "
@@ -294,30 +301,50 @@
 (defn testing-contexts-str
   "Returns a string representation of the current test context. Joins
   strings in *testing-contexts* with spaces."
+  {:added "1.1"}
   []
   (apply str (interpose " " (reverse *testing-contexts*))))
 
 (defn inc-report-counter
   "Increments the named counter in *report-counters*, a ref to a map.
   Does nothing if *report-counters* is nil."
+  {:added "1.1"}
   [name]
   (when *report-counters*
     (dosync (commute *report-counters* assoc name
                      (inc (or (*report-counters* name) 0))))))
 
-
-
 ;;; TEST RESULT REPORTING
 
 (defmulti
-  #^{:doc "Generic reporting function, may be overridden to plug in
+  ^{:doc "Generic reporting function, may be overridden to plug in
    different report formats (e.g., TAP, JUnit).  Assertions such as
    'is' call 'report' to indicate results.  The argument given to
    'report' will be a map with a :type key.  See the documentation at
    the top of test_is.clj for more information on the types of
-   arguments for 'report'."}
+   arguments for 'report'."
+     :dynamic true
+     :added "1.1"}
   report :type)
 
+(defn- file-and-line 
+  [exception depth]
+  (let [^StackTraceElement s (nth (.getStackTrace exception) depth)]
+    {:file (.getFileName s) :line (.getLineNumber s)}))
+
+(defn do-report
+  "Add file and line information to a test result and call report.
+   If you are writing a custom assert-expr method, call this function
+   to pass test results to report."
+  {:added "1.2"}
+  [m]
+  (report
+   (case
+    (:type m)
+    :fail (merge (file-and-line (new java.lang.Throwable) 1) m)
+    :error (merge (file-and-line (:actual m) 0) m) 
+    m)))
+
 (defmethod report :default [m]
   (with-test-out (prn m)))
 
@@ -326,17 +353,17 @@
 
 (defmethod report :fail [m]
   (with-test-out
-   (inc-report-counter :fail)
-   (println "\nFAIL in" (testing-vars-str))
-   (when (seq *testing-contexts*) (println (testing-contexts-str)))
-   (when-let [message (:message m)] (println message))
-   (println "expected:" (pr-str (:expected m)))
-   (println "  actual:" (pr-str (:actual m)))))
+    (inc-report-counter :fail)
+    (println "\nFAIL in" (testing-vars-str m))
+    (when (seq *testing-contexts*) (println (testing-contexts-str)))
+    (when-let [message (:message m)] (println message))
+    (println "expected:" (pr-str (:expected m)))
+    (println "  actual:" (pr-str (:actual m)))))
 
 (defmethod report :error [m]
   (with-test-out
    (inc-report-counter :error)
-   (println "\nERROR in" (testing-vars-str))
+   (println "\nERROR in" (testing-vars-str m))
    (when (seq *testing-contexts*) (println (testing-contexts-str)))
    (when-let [message (:message m)] (println message))
    (println "expected:" (pr-str (:expected m)))
@@ -367,6 +394,7 @@
 
 (defn get-possibly-unbound-var
   "Like var-get but returns nil if the var is unbound."
+  {:added "1.1"}
   [v]
   (try (var-get v)
        (catch IllegalStateException e
@@ -375,6 +403,7 @@
 (defn function?
   "Returns true if argument is a function or a symbol that resolves to
   a function (not a macro)."
+  {:added "1.1"}
   [x]
   (if (symbol? x)
     (when-let [v (resolve x)]
@@ -389,27 +418,29 @@
   'actual' argument will contain the form with all its sub-forms
   evaluated.  If the predicate returns false, the 'actual' form will
   be wrapped in (not...)."
+  {:added "1.1"}
   [msg form]
   (let [args (rest form)
         pred (first form)]
     `(let [values# (list ~@args)
            result# (apply ~pred values#)]
        (if result#
-         (report {:type :pass, :message ~msg,
+         (do-report {:type :pass, :message ~msg,
                   :expected '~form, :actual (cons ~pred values#)})
-         (report {:type :fail, :message ~msg,
+         (do-report {:type :fail, :message ~msg,
                   :expected '~form, :actual (list '~'not (cons '~pred values#))}))
        result#)))
 
 (defn assert-any
   "Returns generic assertion code for any test, including macros, Java
   method calls, or isolated symbols."
+  {:added "1.1"}
   [msg form]
   `(let [value# ~form]
      (if value#
-       (report {:type :pass, :message ~msg,
+       (do-report {:type :pass, :message ~msg,
                 :expected '~form, :actual value#})
-       (report {:type :fail, :message ~msg,
+       (do-report {:type :fail, :message ~msg,
                 :expected '~form, :actual value#}))
      value#))
 
@@ -430,7 +461,7 @@
 
 (defmethod assert-expr :always-fail [msg form]
   ;; nil test: always fail
-  `(report {:type :fail, :message ~msg}))
+  `(do-report {:type :fail, :message ~msg}))
 
 (defmethod assert-expr :default [msg form]
   (if (and (sequential? form) (function? (first form)))
@@ -443,9 +474,9 @@
          object# ~(nth form 2)]
      (let [result# (instance? klass# object#)]
        (if result#
-         (report {:type :pass, :message ~msg,
+         (do-report {:type :pass, :message ~msg,
                   :expected '~form, :actual (class object#)})
-         (report {:type :fail, :message ~msg,
+         (do-report {:type :fail, :message ~msg,
                   :expected '~form, :actual (class object#)}))
        result#)))
 
@@ -456,10 +487,10 @@
   (let [klass (second form)
         body (nthnext form 2)]
     `(try ~@body
-          (report {:type :fail, :message ~msg,
+          (do-report {:type :fail, :message ~msg,
                    :expected '~form, :actual nil})
           (catch ~klass e#
-            (report {:type :pass, :message ~msg,
+            (do-report {:type :pass, :message ~msg,
                      :expected '~form, :actual e#})
             e#))))
 
@@ -467,18 +498,18 @@
   ;; (is (thrown-with-msg? c re expr))
   ;; Asserts that evaluating expr throws an exception of class c.
   ;; Also asserts that the message string of the exception matches
-  ;; (with re-matches) the regular expression re.
+  ;; (with re-find) the regular expression re.
   (let [klass (nth form 1)
         re (nth form 2)
         body (nthnext form 3)]
     `(try ~@body
-          (report {:type :fail, :message ~msg, :expected '~form, :actual nil})
+          (do-report {:type :fail, :message ~msg, :expected '~form, :actual nil})
           (catch ~klass e#
             (let [m# (.getMessage e#)]
-              (if (re-matches ~re m#)
-                (report {:type :pass, :message ~msg,
+              (if (re-find ~re m#)
+                (do-report {:type :pass, :message ~msg,
                          :expected '~form, :actual e#})
-                (report {:type :fail, :message ~msg,
+                (do-report {:type :fail, :message ~msg,
                          :expected '~form, :actual e#})))
             e#))))
 
@@ -486,11 +517,12 @@
 (defmacro try-expr
   "Used by the 'is' macro to catch unexpected exceptions.
   You don't call this."
+  {:added "1.1"}
   [msg form]
   `(try ~(assert-expr msg form)
         (catch Throwable t#
-          (report {:type :error, :message ~msg,
-                   :expected '~form, :actual t#}))))
+          (do-report {:type :error, :message ~msg,
+                      :expected '~form, :actual t#}))))
 
 
 
@@ -511,7 +543,8 @@
 
   (is (thrown-with-msg? c re body)) checks that an instance of c is
   thrown AND that the message on the exception matches (with
-  re-matches) the regular expression re."
+  re-find) the regular expression re."
+  {:added "1.1"} 
   ([form] `(is ~form nil))
   ([form msg] `(try-expr ~msg ~form)))
 
@@ -528,12 +561,14 @@
                (is (= 4 (* 2 2))))
 
   Note: This breaks some reporting features, such as line numbers."
+  {:added "1.1"}
   [argv expr & args]
   `(temp/do-template ~argv (is ~expr) ~@args))
 
 (defmacro testing
   "Adds a new string to the list of testing contexts.  May be nested,
   but must occur inside a test function (deftest)."
+  {:added "1.1"}
   [string & body]
   `(binding [*testing-contexts* (conj *testing-contexts* ~string)]
      ~@body))
@@ -548,6 +583,7 @@
 
   When *load-tests* is false, only evaluates the definition, ignoring
   the tests."
+  {:added "1.1"}
   [definition & body]
   (if *load-tests*
     `(doto ~definition (alter-meta! assoc :test (fn [] ~@body)))
@@ -565,6 +601,7 @@
   itself.
 
   When *load-tests* is false, deftest is ignored."
+  {:added "1.1"}
   [name & body]
   (when *load-tests*
     `(def ~(vary-meta name assoc :test `(fn [] ~@body))
@@ -572,6 +609,7 @@
 
 (defmacro deftest-
   "Like deftest but creates a private var."
+  {:added "1.1"}
   [name & body]
   (when *load-tests*
     `(def ~(vary-meta name assoc :test `(fn [] ~@body) :private true)
@@ -584,6 +622,7 @@
   The var must already exist.  Does not modify the value of the var.
 
   When *load-tests* is false, set-test is ignored."
+  {:added "1.1"}
   [name & body]
   (when *load-tests*
     `(alter-meta! (var ~name) assoc :test (fn [] ~@body))))
@@ -595,6 +634,7 @@
 (defn- add-ns-meta
   "Adds elements in coll to the current namespace metadata as the
   value of key."
+  {:added "1.1"}
   [key coll]
   (alter-meta! *ns* assoc key coll))
 
@@ -602,6 +642,7 @@
   "Wrap test runs in a fixture function to perform setup and
   teardown. Using a fixture-type of :each wraps every test
   individually, while:once wraps the whole run in a single function."
+  {:added "1.1"}
   (fn [fixture-type & args] fixture-type))
 
 (defmethod use-fixtures :each [fixture-type & args]
@@ -612,18 +653,21 @@
 
 (defn- default-fixture
   "The default, empty, fixture function.  Just calls its argument."
+  {:added "1.1"}
   [f]
   (f))
 
 (defn compose-fixtures
   "Composes two fixture functions, creating a new fixture function
   that combines their behavior."
+  {:added "1.1"}
   [f1 f2]
   (fn [g] (f1 (fn [] (f2 g)))))
 
 (defn join-fixtures
   "Composes a collection of fixtures, in order.  Always returns a valid
   fixture function, even if the collection is empty."
+  {:added "1.1"}
   [fixtures]
   (reduce compose-fixtures default-fixture fixtures))
 
@@ -635,19 +679,21 @@
 (defn test-var
   "If v has a function in its :test metadata, calls that function,
   with *testing-vars* bound to (conj *testing-vars* v)."
+  {:dynamic true, :added "1.1"}
   [v]
   (when-let [t (:test (meta v))]
     (binding [*testing-vars* (conj *testing-vars* v)]
-      (report {:type :begin-test-var, :var v})
+      (do-report {:type :begin-test-var, :var v})
       (inc-report-counter :test)
       (try (t)
            (catch Throwable e
-             (report {:type :error, :message "Uncaught exception, not in assertion."
+             (do-report {:type :error, :message "Uncaught exception, not in assertion."
                       :expected nil, :actual e})))
-      (report {:type :end-test-var, :var v}))))
+      (do-report {:type :end-test-var, :var v}))))
 
 (defn test-all-vars
   "Calls test-var on every var interned in the namespace, with fixtures."
+  {:added "1.1"}
   [ns]
   (let [once-fixture-fn (join-fixtures (::once-fixtures (meta ns)))
         each-fixture-fn (join-fixtures (::each-fixtures (meta ns)))]
@@ -665,16 +711,17 @@
   Internally binds *report-counters* to a ref initialized to
   *inital-report-counters*.  Returns the final, dereferenced state of
   *report-counters*."
+  {:added "1.1"}
   [ns]
   (binding [*report-counters* (ref *initial-report-counters*)]
     (let [ns-obj (the-ns ns)]
-      (report {:type :begin-test-ns, :ns ns-obj})
+      (do-report {:type :begin-test-ns, :ns ns-obj})
       ;; If the namespace has a test-ns-hook function, call that:
       (if-let [v (find-var (symbol (str (ns-name ns-obj)) "test-ns-hook"))]
 	((var-get v))
         ;; Otherwise, just test every var in the namespace.
         (test-all-vars ns-obj))
-      (report {:type :end-test-ns, :ns ns-obj}))
+      (do-report {:type :end-test-ns, :ns ns-obj}))
     @*report-counters*))
 
 
@@ -685,11 +732,12 @@
   "Runs all tests in the given namespaces; prints results.
   Defaults to current namespace if none given.  Returns a map
   summarizing test results."
+  {:added "1.1"}
   ([] (run-tests *ns*))
   ([& namespaces]
      (let [summary (assoc (apply merge-with + (map test-ns namespaces))
                      :type :summary)]
-       (report summary)
+       (do-report summary)
        summary)))
 
 (defn run-all-tests
@@ -697,12 +745,14 @@
   Optional argument is a regular expression; only namespaces with
   names matching the regular expression (with re-matches) will be
   tested."
+  {:added "1.1"}
   ([] (apply run-tests (all-ns)))
   ([re] (apply run-tests (filter #(re-matches re (name (ns-name %))) (all-ns)))))
 
 (defn successful?
   "Returns true if the given test summary indicates all tests
   were successful, false otherwise."
+  {:added "1.1"}
   [summary]
-  (and (zero? (:fail summary))
-       (zero? (:error summary))))
+  (and (zero? (:fail summary 0))
+       (zero? (:error summary 0))))
