@@ -16,6 +16,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.Callable;
 import java.util.*;
 import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 import java.io.*;
 import java.lang.reflect.Array;
 import java.math.BigDecimal;
@@ -180,10 +181,10 @@ final static Keyword TAG_KEY = Keyword.intern(null, "tag");
 final static public Var AGENT = Var.intern(CLOJURE_NS, Symbol.create("*agent*"), null);
 final static public Var READEVAL = Var.intern(CLOJURE_NS, Symbol.create("*read-eval*"), T);
 final static public Var ASSERT = Var.intern(CLOJURE_NS, Symbol.create("*assert*"), T);
-final static public Var MACRO_META = Var.intern(CLOJURE_NS, Symbol.create("*macro-meta*"), null);
 final static public Var MATH_CONTEXT = Var.intern(CLOJURE_NS, Symbol.create("*math-context*"), null);
 static Keyword LINE_KEY = Keyword.intern(null, "line");
 static Keyword FILE_KEY = Keyword.intern(null, "file");
+static Keyword DECLARED_KEY = Keyword.intern(null, "declared");
 final static public Var USE_CONTEXT_CLASSLOADER =
 		Var.intern(CLOJURE_NS, Symbol.create("*use-context-classloader*"), T);
 //final static public Var CURRENT_MODULE = Var.intern(Symbol.create("clojure.core", "current-module"),
@@ -219,6 +220,15 @@ final static IFn inNamespace = new AFn(){
 	}
 };
 
+final static IFn bootNamespace = new AFn(){
+	public Object invoke(Object __form, Object __env,Object arg1) throws Exception{
+		Symbol nsname = (Symbol) arg1;
+		Namespace ns = Namespace.findOrCreate(nsname);
+		CURRENT_NS.set(ns);
+		return ns;
+	}
+};
+
 public static List<String> processCommandLine(String[] args){
 	List<String> arglist = Arrays.asList(args);
 	int split = arglist.indexOf("--");
@@ -229,12 +239,30 @@ public static List<String> processCommandLine(String[] args){
 	return arglist;
 }
 
+// duck typing stderr plays nice with e.g. swank 
+public static PrintWriter errPrintWriter(){
+    Writer w = (Writer) ERR.deref();
+    if (w instanceof PrintWriter) {
+        return (PrintWriter) w;
+    } else {
+        return new PrintWriter(w);
+    }
+}
+
 static public final Object[] EMPTY_ARRAY = new Object[]{};
-static public final Comparator DEFAULT_COMPARATOR = new Comparator(){
-	public int compare(Object o1, Object o2){
+static public final Comparator DEFAULT_COMPARATOR = new DefaultComparator();
+
+private static final class DefaultComparator implements Comparator, Serializable {
+    public int compare(Object o1, Object o2){
 		return Util.compare(o1, o2);
 	}
-};
+
+    private Object readResolve() throws ObjectStreamException {
+        // ensures that we aren't hanging onto a new default comparator for every
+        // sorted set, etc., we deserialize
+        return DEFAULT_COMPARATOR;
+    }
+}
 
 static AtomicInteger id = new AtomicInteger(1);
 
@@ -247,14 +275,6 @@ static public void addURL(Object url) throws Exception{
 		throw new IllegalAccessError("Context classloader is not a DynamicClassLoader");
 }
 
-final static public Object EOS = new Object();
-final static public Object SKIP = new Object();
-static final public IFn EMPTY_GEN = new AFn(){
-    synchronized public Object invoke() throws Exception {
-        return EOS;
-    }
-};
-
 static{
 	Keyword dockw = Keyword.intern(null, "doc");
 	Keyword arglistskw = Keyword.intern(null, "arglists");
@@ -264,8 +284,7 @@ static{
 	AGENT.setMeta(map(dockw, "The agent currently running an action on this thread, else nil"));
 	AGENT.setTag(Symbol.create("clojure.lang.Agent"));
 	MATH_CONTEXT.setTag(Symbol.create("java.math.MathContext"));
-	//during bootstrap ns same as in-ns
-	Var nv = Var.intern(CLOJURE_NS, NAMESPACE, inNamespace);
+	Var nv = Var.intern(CLOJURE_NS, NAMESPACE, bootNamespace);
 	nv.setMacro();
 	Var v;
 	v = Var.intern(CLOJURE_NS, IN_NAMESPACE, inNamespace);
@@ -279,15 +298,6 @@ static{
 	               });
 	v.setMeta(map(dockw, "Sequentially read and evaluate the set of forms contained in the file.",
 	              arglistskw, list(vector(namesym))));
-	v = Var.intern(CLOJURE_NS, IDENTICAL,
-	               new AFn(){
-		               public Object invoke(Object arg1, Object arg2)
-				               throws Exception{
-			               return arg1 == arg2 ? RT.T : RT.F;
-		               }
-	               });
-	v.setMeta(map(dockw, "Tests if 2 arguments are the same object",
-	              arglistskw, list(vector(Symbol.create("x"), Symbol.create("y")))));
 	try {
 		doInit();
 	}
@@ -339,7 +349,7 @@ public static void loadResourceScript(Class c, String name, boolean failIfNotFou
 }
 
 static public void init() throws Exception{
-	((PrintWriter) RT.ERR.deref()).println("No need to call RT.init() anymore");
+	RT.errPrintWriter().println("No need to call RT.init() anymore");
 }
 
 static public long lastModified(URL url, String libfile) throws Exception{
@@ -442,24 +452,6 @@ static public ISeq seq(Object coll){
 		return seqFrom(coll);
 }
 
-static public Stream stream(final Object coll) throws Exception{
-	if(coll == null)
-		return new Stream(EMPTY_GEN);
-	else if(coll instanceof Streamable)
-		return ((Streamable) coll).stream();
-	else if(coll instanceof Fn)
-		return new Stream((IFn) coll);
-	else if(coll instanceof Iterable)
-		return new Stream(new IteratorStream(((Iterable) coll).iterator()));
-	else if(coll.getClass().isArray())
-		return ArrayStream.createFromObject(coll);
-	else if(coll instanceof String)
-		return ArrayStream.createFromObject(((String) coll).toCharArray());
-	else
-	    return new Stream(new ASeq.Src(RT.seq(coll)));
-
-}
-
 static ISeq seqFrom(Object coll){
 	if(coll instanceof Seqable)
 		return ((Seqable) coll).seq();
@@ -469,8 +461,8 @@ static ISeq seqFrom(Object coll){
 		return IteratorSeq.create(((Iterable) coll).iterator());
 	else if(coll.getClass().isArray())
 		return ArraySeq.createFromObject(coll);
-	else if(coll instanceof String)
-		return StringSeq.create((String) coll);
+	else if(coll instanceof CharSequence)
+		return StringSeq.create((CharSequence) coll);
 	else if(coll instanceof Map)
 		return seq(((Map) coll).entrySet());
 	else {
@@ -494,15 +486,13 @@ static public IPersistentMap meta(Object x){
 	return null;
 }
 
-public static int count(Counted o){
-	if(o != null)
-		return o.count();
-	return 0;
-}
-
 public static int count(Object o){
 	if(o instanceof Counted)
 		return ((Counted) o).count();
+	return countFrom(Util.ret1(o, o = null));
+}
+
+static int countFrom(Object o){
 	if(o == null)
 		return 0;
 	else if(o instanceof IPersistentCollection) {
@@ -516,8 +506,8 @@ public static int count(Object o){
 		}
 		return i;
 	}
-	else if(o instanceof String)
-		return ((String) o).length();
+	else if(o instanceof CharSequence)
+		return ((CharSequence) o).length();
 	else if(o instanceof Collection)
 		return ((Collection) o).size();
 	else if(o instanceof Map)
@@ -613,10 +603,14 @@ static public Object pop(Object x){
 }
 
 static public Object get(Object coll, Object key){
+	if(coll instanceof ILookup)
+		return ((ILookup) coll).valAt(key);
+	return getFrom(coll, key);
+}
+
+static Object getFrom(Object coll, Object key){
 	if(coll == null)
 		return null;
-	else if(coll instanceof ILookup)
-		return ((ILookup) coll).valAt(key);
 	else if(coll instanceof Map) {
 		Map m = (Map) coll;
 		return m.get(key);
@@ -636,10 +630,14 @@ static public Object get(Object coll, Object key){
 }
 
 static public Object get(Object coll, Object key, Object notFound){
+	if(coll instanceof ILookup)
+		return ((ILookup) coll).valAt(key, notFound);
+	return getFrom(coll, key, notFound);
+}
+
+static Object getFrom(Object coll, Object key, Object notFound){
 	if(coll == null)
 		return notFound;
-	else if(coll instanceof ILookup)
-		return ((ILookup) coll).valAt(key, notFound);
 	else if(coll instanceof Map) {
 		Map m = (Map) coll;
 		if(m.containsKey(key))
@@ -721,10 +719,14 @@ static public Object dissoc(Object coll, Object key) throws Exception{
 static public Object nth(Object coll, int n){
 	if(coll instanceof Indexed)
 		return ((Indexed) coll).nth(n);
+	return nthFrom(Util.ret1(coll, coll = null), n);
+}
+
+static Object nthFrom(Object coll, int n){
 	if(coll == null)
 		return null;
-	else if(coll instanceof String)
-		return Character.valueOf(((String) coll).charAt(n));
+	else if(coll instanceof CharSequence)
+		return Character.valueOf(((CharSequence) coll).charAt(n));
 	else if(coll.getClass().isArray())
 		return Reflector.prepRet(Array.get(coll, n));
 	else if(coll instanceof RandomAccess)
@@ -758,17 +760,19 @@ static public Object nth(Object coll, int n){
 static public Object nth(Object coll, int n, Object notFound){
 	if(coll instanceof Indexed) {
 		Indexed v = (Indexed) coll;
-		if(n >= 0 && n < v.count())
-			return v.nth(n);
-		return notFound;
+			return v.nth(n, notFound);
 	}
-	else if(coll == null)
+	return nthFrom(coll, n, notFound);
+}
+
+static Object nthFrom(Object coll, int n, Object notFound){
+	if(coll == null)
 		return notFound;
 	else if(n < 0)
 		return notFound;
 
-	else if(coll instanceof String) {
-		String s = (String) coll;
+	else if(coll instanceof CharSequence) {
+		CharSequence s = (CharSequence) coll;
 		if(n < s.length())
 			return Character.valueOf(s.charAt(n));
 		return notFound;
@@ -877,7 +881,12 @@ static public Number box(double x){
 static public char charCast(Object x){
 	if(x instanceof Character)
 		return ((Character) x).charValue();
-	return (char) ((Number) x).intValue();
+
+	long n = ((Number) x).longValue();
+	if(n < Character.MIN_VALUE || n > Character.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for char: " + x);
+
+	return (char) n;
 }
 
 static public boolean booleanCast(Object x){
@@ -886,17 +895,31 @@ static public boolean booleanCast(Object x){
 	return x != null;
 }
 
+static public boolean booleanCast(boolean x){
+	return x;
+}
+
 static public byte byteCast(Object x){
-	return ((Number) x).byteValue();
+	long n = ((Number) x).longValue();
+	if(n < Byte.MIN_VALUE || n > Byte.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for byte: " + x);
+
+	return (byte) n;
 }
 
 static public short shortCast(Object x){
-	return ((Number) x).shortValue();
+	long n = ((Number) x).longValue();
+	if(n < Short.MIN_VALUE || n > Short.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for short: " + x);
+
+	return (short) n;
 }
 
 static public int intCast(Object x){
+	if(x instanceof Integer)
+		return ((Integer)x).intValue();
 	if(x instanceof Number)
-		return ((Number) x).intValue();
+		return intCast(((Number) x).longValue());
 	return ((Character) x).charValue();
 }
 
@@ -917,14 +940,20 @@ static public int intCast(int x){
 }
 
 static public int intCast(float x){
+	if(x < Integer.MIN_VALUE || x > Integer.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for int: " + x);
 	return (int) x;
 }
 
 static public int intCast(long x){
+	if(x < Integer.MIN_VALUE || x > Integer.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for int: " + x);
 	return (int) x;
 }
 
 static public int intCast(double x){
+	if(x < Integer.MIN_VALUE || x > Integer.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for int: " + x);
 	return (int) x;
 }
 
@@ -937,6 +966,8 @@ static public long longCast(int x){
 }
 
 static public long longCast(float x){
+	if(x < Long.MIN_VALUE || x > Long.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for long: " + x);
 	return (long) x;
 }
 
@@ -945,11 +976,21 @@ static public long longCast(long x){
 }
 
 static public long longCast(double x){
+	if(x < Long.MIN_VALUE || x > Long.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for long: " + x);
 	return (long) x;
 }
 
 static public float floatCast(Object x){
-	return ((Number) x).floatValue();
+	if(x instanceof Float)
+		return ((Float) x).floatValue();
+
+	double n = ((Number) x).doubleValue();
+	if(n < -Float.MAX_VALUE || n > Float.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for float: " + x);
+
+	return (float) n;
+
 }
 
 static public float floatCast(int x){
@@ -965,6 +1006,9 @@ static public float floatCast(long x){
 }
 
 static public float floatCast(double x){
+	if(x < -Float.MAX_VALUE || x > Float.MAX_VALUE)
+		throw new IllegalArgumentException("Value out of range for float: " + x);
+	
 	return (float) x;
 }
 
@@ -992,12 +1036,12 @@ static public IPersistentMap map(Object... init){
 	if(init == null)
 		return PersistentArrayMap.EMPTY;
 	else if(init.length <= PersistentArrayMap.HASHTABLE_THRESHOLD)
-		return new PersistentArrayMap(init);
-	return PersistentHashMap.create(init);
+		return PersistentArrayMap.createWithCheck(init);
+	return PersistentHashMap.createWithCheck(init);
 }
 
 static public IPersistentSet set(Object... init){
-	return PersistentHashSet.create(init);
+	return PersistentHashSet.createWithCheck(init);
 }
 
 static public IPersistentVector vector(Object... init){
@@ -1068,6 +1112,20 @@ static public ISeq arrayToList(Object[] a) throws Exception{
 	return ret;
 }
 
+static public Object[] object_array(Object sizeOrSeq){
+	if(sizeOrSeq instanceof Number)
+		return new Object[((Number) sizeOrSeq).intValue()];
+	else
+		{
+		ISeq s = RT.seq(sizeOrSeq);
+		int size = RT.count(s);
+		Object[] ret = new Object[size];
+		for(int i = 0; i < size && s != null; i++, s = s.next())
+			ret[i] = s.first();
+		return ret;
+		}
+}
+
 static public Object[] toArray(Object coll) throws Exception{
 	if(coll == null)
 		return EMPTY_ARRAY;
@@ -1345,6 +1403,10 @@ static public void print(Object x, Writer w) throws Exception{
 			Var v = (Var) x;
 			w.write("#=(var " + v.ns.name + "/" + v.sym + ")");
 		}
+		else if(x instanceof Pattern) {
+			Pattern p = (Pattern) x;
+			w.write("#\"" + p.pattern() + "\"");
+		}
 		else w.write(x.toString());
 	}
 	//*/
@@ -1669,17 +1731,5 @@ static public Object[] aclone(Object[] xs){
 	return xs.clone();
 }
 
-static public Object aget(Object xs, int i){
-	return Reflector.prepRet(Array.get(xs, i));
-}
-
-static public Object aset(Object xs, int i, Object v){
-	Array.set(xs, i, v);
-	return v;
-}
-
-static public int alength(Object xs){
-	return Array.getLength(xs);
-}
 
 }
