@@ -128,6 +128,8 @@ public boolean isBound(){
 }
 
 final public Object get(){
+	if(count.get() == 0 && root != dvals)
+		return root;
 	return deref();
 }
 
@@ -208,6 +210,10 @@ public Object getRoot(){
 	throw new IllegalStateException(String.format("Var %s/%s is unbound.", ns, sym));
 }
 
+public Object getRawRoot(){
+		return root;
+}
+
 public Object getTag(){
 	return meta().valAt(RT.TAG_KEY);
 }
@@ -234,7 +240,7 @@ synchronized public void bindRoot(Object root){
 	this.root = root;
     try
         {
-        alterMeta(assoc, RT.list(macroKey, RT.F));
+        alterMeta(dissoc, RT.list(macroKey));
         }
     catch (Exception e)
         {
@@ -322,7 +328,7 @@ public static Associative getThreadBindings(){
 	return ret;
 }
 
-final Box getThreadBinding(){
+public final Box getThreadBinding(){
 	if(count.get() > 0)
 		{
 		IMapEntry e = dvals.get().bindings.entryAt(this);
@@ -482,4 +488,10 @@ static IFn assoc = new AFn(){
         return RT.assoc(m, k, v);
     }
 };
+static IFn dissoc = new AFn() {
+    @Override
+    public Object invoke(Object c, Object k) throws Exception {
+        return RT.dissoc(c, k);
+    }
+};
 }
