@@ -517,6 +517,35 @@
       {} {:a 1 :b 2}
       #{} #{1 2} ))
 
+(deftest test-get
+  (let [m {:a 1, :b 2, :c {:d 3, :e 4}, :f nil, :g false, nil {:h 5}}]
+    (is (thrown? IllegalArgumentException (get-in {:a 1} 5)))
+    (are [x y] (= x y)
+         (get m :a) 1
+         (get m :e) nil
+         (get m :e 0) 0
+         (get m :b 0) 2
+         (get m :f 0) nil
+
+         (get-in m [:c :e]) 4
+         (get-in m '(:c :e)) 4
+         (get-in m [:c :x]) nil
+         (get-in m [:f]) nil
+         (get-in m [:g]) false
+         (get-in m [:h]) nil
+         (get-in m []) m
+         (get-in m nil) m
+
+         (get-in m [:c :e] 0) 4
+         (get-in m '(:c :e) 0) 4
+         (get-in m [:c :x] 0) 0
+         (get-in m [:b] 0) 2
+         (get-in m [:f] 0) nil
+         (get-in m [:g] 0) false
+         (get-in m [:h] 0) 0
+         (get-in m [:x :y] {:y 1}) {:y 1}
+         (get-in m [] 0) m
+         (get-in m nil 0) m)))
 
 ;; *** Sets ***
 
@@ -534,23 +563,6 @@
       (hash-set 1 2) (hash-set 2 1)
       (hash-set 3 1 2) (hash-set 1 2 3) )
 
-  ; equal and unique
-  (are [x] (and (= (hash-set x)   #{x})
-                (= (hash-set x x) #{x}))
-      nil
-      false true
-      0 42
-      0.0 3.14
-      2/3
-      0M 1M
-      \c
-      "" "abc"
-      'sym
-      :kw
-      () '(1 2)
-      [] [1 2]
-      {} {:a 1 :b 2}
-      #{} #{1 2} )
 
   (are [x y] (= x y)
       ; equal classes
@@ -745,6 +757,7 @@
 
   ; identity
   (are [x] (= (disj x) x)
+      nil
       #{}
       #{1 2 3}
       ; different data types
@@ -770,6 +783,9 @@
       (sorted-set 1 2) )
 
   (are [x y] (= x y)
+      (disj nil :a) nil
+      (disj nil :a :b) nil
+
       (disj #{} :a) #{}
       (disj #{} :a :b) #{}
 
@@ -791,3 +807,24 @@
       (disj #{#{nil}} #{nil}) #{} ))
 
 
+;; *** Queues ***
+
+(deftest test-queues
+  (let [EMPTY clojure.lang.PersistentQueue/EMPTY]
+    (are [x y] (= x y)
+      EMPTY EMPTY
+      (into EMPTY (range 50)) (into EMPTY (range 50))
+      (range 5) (into EMPTY (range 5))
+      (range 1 6) (-> EMPTY
+                    (into (range 6))
+                    pop))
+    (are [x y] (not= x y)
+      (range 5) (into EMPTY (range 6))
+      (range 6) (into EMPTY (range 5))
+      (range 0 6) (-> EMPTY
+                    (into (range 6))
+                    pop)
+      (range 1 6) (-> EMPTY
+                    (into (range 7))
+                    pop))))
+
